<?php 
// edit.php
//
// Copyright 2004, David Barrett.  All Rights Reserved.
//   Email: dbarrett@quinthar.com
//     Web: http://www.quinthar.com
//
// See LICENSE for the complete licensing details.
require_once( "_global.php" );

////////////////////////////////////////////////////////////////////////////////
///////////////////////////////// Disable Caching //////////////////////////////
////////////////////////////////////////////////////////////////////////////////
QWDisableCaching( );


////////////////////////////////////////////////////////////////////////////////
////////////////////////// Force Links to Open New Window //////////////////////
////////////////////////////////////////////////////////////////////////////////
$QW['hrefTarget'] = " TARGET='_blank'";

////////////////////////////////////////////////////////////////////////////////
////////////////////////// Guard Against Orphan Pages //////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Verify this page either exists, or is being spawned from a page that exists,
// or the user is the admin
if( !$QW['userIsAuthenticated'] && 
    !is_file( $QW['pagePath'] ) && 
    (!is_file( $QW['fromPagePath'] ) || is_file( $QW['fromPassPath'] )) )
{
	// Prompt to login
	QWRedirect( "login.php" );
	exit;
}

////////////////////////////////////////////////////////////////////////////////
///////////////////////////////// Access Control ///////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Verify there's actually a page set
if( !$QW['requestPage'] )
{
	// Must first log onto this page -- QWRedirect to login
	QWRedirect( "index.php" );
	exit;
}

// A pasword for this page exists, see if we can authenticate to it
if( QWHoldsLockfile( $QW['globalLockPath'], $QW['lockPath'], $HTTP_SERVER_VARS['REMOTE_ADDR'] ) )
{
	// Already authenticated and owns the lock
}
else if( !$QW['pageIsProtected'] || $QW['userIsAuthenticated'] )
{
	// Authenticated (or not needed), try to get the edit lockfile
	if( QWCreateLockfile( $QW['globalLockPath'], $QW['lockPath'], $HTTP_SERVER_VARS['REMOTE_ADDR'] ) )
	{
		// Now authenticated and with the lockfile
	}
	else
	{
		// Authenticated, but someone else is editing -- QWRedirect to index
		QWRedirect( "index.php?page=$QW[page]$QW[URLSuffix]" );
		exit;
	}
}
else
{
	// Not authenticated
	QWRedirect( "login.php?page=$QW[page]$QW[URLSuffix]" );
	exit;
}

////////////////////////////////////////////////////////////////////////////////
/////////////////////////////// Process Actions ////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
$wikiData = "";
if( $QW['requestAction'] == "Save" || $QW['requestAction'] == "Done" )
{
	// Update the checked value
	if( $QW['userIsAuthenticated'] && $QW['requestProtect'] != $QW['pageIsProtected'] )
	{
		// Create or delete the .pass file for this page
		if( $QW['requestProtect'] )           touch( $QW['passPath'] );
		else if( is_file( $QW['passPath'] ) ) unlink( $QW['passPath'] );
		$QW['pageIsProtected'] = $QW['requestProtect'];
	}	
	
	// Save the page, if there's any data, else delete it
	if( strlen( $QW['requestWikiData'] ) )
	{
		// Strip out forbidden HTML
		$wikiData = strip_tags( $QW['requestWikiData'], $QW_CONFIG['acceptableHTMLTags'] );
		
		// Save the updated data
		$fp = fopen( $QW['pagePath'], "wb" );
		fwrite( $fp, $wikiData, strlen( $wikiData ) );
		fclose( $fp );
	}
	else
	{
		// Just delete the file, and any associated password
		if( is_file( $QW['pagePath'] ) ) unlink( $QW['pagePath'] );
		if( is_file( $QW['passPath'] ) ) unlink( $QW['passPath'] );
	}
}

if( $QW['requestAction'] == "Done" || $QW['requestAction'] == "Cancel" )
{	
	// Make a backup of the latest version
	$backupDir = 'backups/';
	QWVerifyDirectory( $backupDir );
	$c = 0;
	do
	{
		// Increment the version number
		$backupPath = sprintf( "$backupDir/$QW[page].%04d.qwiki", $c++ );
	}
	while( is_file( $backupPath ) );
	if( is_file( $QW['pagePath'] ) ) copy( $QW['pagePath'], $backupPath );
	
	// Notify the mailing list, if requested
	if( $QW_CONFIG['emailEnable'] && $QW['requestNotify'] && $QW['requestAction'] == "Done" )
	{
		// Mailing list patterns
		// **NOTE** If you add more match/replace patterns, be sure put longer matching patterns
		//          before shorter (such as QW_PAGELINK before QW_PAGE)
		$matchPatternArray = array(
			'/QW_HOMELINK/',
			'/QW_PAGELINK/',
			'/QW_PAGE/',
			'/QW_UNSUBSCRIBELINK/',
			'/QW_RESUBSCRIBELINK/',
			'/QW_BODYTEXT/',
			'/QW_BODYHTML/',
			'/QW_CSS/',
			'/QW_ADMINSIG/',
			'/QW_TITLE/'
		);
		$replacePatternArray = array(
			$QW['homeLink'],
			"$QW[homeLink]/index.php?page=$QW[page]",
			$QW['page'],
			"$QW[homeLink]/mailinglist.php?page=$QW[page]&action=unsubscribe&email=QW_MAILINGLISTEMAIL",
			"$QW[homeLink]/mailinglist.php?page=$QW[page]&action=subscribe&email=QW_MAILINGLISTEMAIL",
			strip_tags( QWGetFileContents( $QW['pagePath'] ) ),
			QWFormatQwikiFile( $QW['pagePath'] ),
			QWGetFileContents( $QW_CONFIG['stylesheetFilename'] ),
			$QW_CONFIG['adminSig'],
			$QW_CONFIG['title']
		);	
		
		// Compose the HTML and text messages
		$subject = preg_replace( $matchPatternArray, $replacePatternArray, $QW_CONFIG['emailChangedSubject'] );
		$textBody = preg_replace( $matchPatternArray, $replacePatternArray, QWGetFileContents( $QW_CONFIG['emailChangedBodyTextFilename'] ) );
		$htmlBody = preg_replace( $matchPatternArray, $replacePatternArray, QWGetFileContents( $QW_CONFIG['emailChangedBodyHTMLFilename'] ) );
		QWMailMailingList( $QW['mailPath'], $QW_CONFIG['emailFrom'], $subject, $textBody, $htmlBody );
	}
		
	// Delete the lock file
	if( file_exists( $QW['lockPath'] ) ) unlink( $QW['lockPath'] );
	QWRedirect( "index.php?page=$QW[page]$QW[URLSuffix]" );
	exit;
}
else if( ini_get( 'file_uploads' ) && $QW_CONFIG['enableAttach'] && $QW['requestAction'] == "Attach" )
{
	// Verify the correct user is attempting to upload a file to this page
	if( QWHoldsLockfile( $QW['globalLockPath'], $QW['lockPath'], $HTTP_SERVER_VARS['REMOTE_ADDR'] ) )
	{
		// Verfify there is a file, it's not too big
		if( $HTTP_POST_FILES[ 'userfile' ][ 'size' ] != 0 && 
		    $HTTP_POST_FILES[ 'userfile' ][ 'size' ] <= $QW_CONFIG['maxUploadSize'] )
		{
			// Verify it's an accepted filetype
			$pathParts = pathinfo( $HTTP_POST_FILES[ 'userfile' ][ 'name' ] );
			$extension = $pathParts['extension'];
			$accept = false;
			foreach( $QW_CONFIG['acceptableExtensions'] as $testExtension )
				if( !strcasecmp( $testExtension, $extension ) )
				{
					// Accept this file
					$accept = true;
					break;
				}
				
			// If it's acceptable, copy the file
			if( $accept )
			{
				// Create this page's attachement directory
				if( !($errorHTML = QWVerifyDirectory( $QW['attachDir'] )) )
				{
					// Move the uploaded file into the directory
					$tempFilename= $HTTP_POST_FILES[ 'userfile' ][ 'tmp_name' ];
					$destFilename = $QW['attachDir'] . "/" . $HTTP_POST_FILES[ 'userfile' ][ 'name' ];
					move_uploaded_file( $tempFilename, $destFilename );
				}
			}
			else
			{
				// Invalid extension
				$errorHTML = "Error - Attachment failed because <B>'.$extension'</B> is a blocked extension.  Only <B>" . 
				             QWCollapseArray( $QW_CONFIG['acceptableExtensions'], ", ", "or " ) . "</B> are allowed." ;
			}
		}
	}
}
else if( ini_get( 'file_uploads' ) && $QW_CONFIG['enableAttach'] && $QW['requestAction'] == "deletefile" )
{
	// Verify the correct user is attempting to upload a file to this page
	if( QWHoldsLockfile( $QW['globalLockPath'], $QW['lockPath'], $HTTP_SERVER_VARS['REMOTE_ADDR'] ) )
	{
		// Remove any paths added to the string
		$pathParts = pathinfo( $QW['requestFilename'] );
		$path = "$QW[attachDir]/$pathParts[basename]";
		if( is_file( $path ) )
		{
			// Delete the file
			unlink( $path );
		}
	}
}



////////////////////////////////////////////////////////////////////////////////
///////////////////////////////// Output Page //////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Attempt to load the existing file
$titlePrefix = "";
if( file_exists( $QW['pagePath'] ) )
{
	// Load the file, if need be
	if( !$wikiData )
	{
		// Load the data
		$fp = fopen( $QW['pagePath'], "rb" );
		$wikiData = fread( $fp, filesize( $QW['pagePath'] ) );
		fclose( $fp );
	}
	$titlePrefix = "Editing: ";
}
else
{
	// Set the title
	$titlePrefix = "Creating: ";
}

////////////////////////////////////////////////////////////////////////////////
//////////////////////////// Define Template Functions /////////////////////////
////////////////////////////////////////////////////////////////////////////////

// QWTEditFormatTitle
$QW_TEMPLATE['getTitle'] = "QWTEditFormatTitle";
function QWTEditFormatTitle( )
{
	// Output a title
	global $titlePrefix, $QW;
	return $titlePrefix . QWCleanQwikiPageName( $QW['page'] );
}

// QWTEditFormatBody
$QW_TEMPLATE['injectBody'] = "QWTEditInjectBody";
function QWTEditInjectBody( )
{
	global $QW, $wikiData, $QW_CONFIG, $errorHTML;
	
	// Output the error, if there is any
	if( $errorHTML ) echo QWFormatMessage( $errorHTML );
	
	?>
	<DIV ALIGN='center'>
		<FORM CLASS='QWForm' ENCTYPE="multipart/form-data" METHOD="post" ACTION="edit.php">
			<TABLE CLASS='QWInnerSection'>
				<TR CLASS="QWInnerSectionTitle"><TD>Page Source</TD></TR>
				<INPUT TYPE="hidden" NAME="page" VALUE="<?php echo $QW['page'] ?>"/>
				<INPUT TYPE="hidden" NAME="from" VALUE="<?php echo $QW['requestFrom'] ?>"/>
				<INPUT TYPE="hidden" NAME="debug" VALUE="<?php echo $QW['requestDebug'] ?>"/>
				<INPUT TYPE="hidden" NAME="help" VALUE="<?php echo $QW['requestHelp'] ?>"/>
				<TR><TD ALIGN='center'><TEXTAREA NAME="wikidata" COLS="80" ROWS="20"><?php echo htmlentities( $wikiData ) ?></TEXTAREA></TD></TR>
				<TR><TD ALIGN='center'><INPUT TYPE='submit' NAME='action' ACTION='edit.php' VALUE='Done'/><INPUT TYPE='submit' NAME='action' ACTION='edit.php' VALUE='Save'/><INPUT TYPE='submit' NAME='action' ACTION='edit.php' VALUE='Cancel'/></TD></TR>
				<?php if( is_file( $QW['mailPath'] ) ) { ?> 
					<TR><TD ALIGN='center'><INPUT TYPE="checkbox" NAME="notify" checked/> Notify the <B><?php echo $QW['page'] ?></B> mailing list when done</TD></TR>
				<?php } ?>
				<?php if( $QW['userIsAuthenticated'] ) {
					if( $QW_CONFIG['globalEditLock'] ) {
						// Page cannot be individually locked due to global edit lock
						?>
							<INPUT TYPE='hidden' NAME='protect' VALUE='<?php echo $QW['pageIsProtected'] ?>'/>
							<TR><TD ALIGN='center'>(Only admins can edit <B><?php echo $QW['page'] ?></B> because global edit locking is <B>enabled</B>)</TD></TR>
						<?php
					}
					else
					{
						// Allow the page to be individually locked
						$isChecked = ( $QW['pageIsProtected'] ? "checked" : "" );
						?><TR><TD ALIGN='center'><INPUT <?php echo $isChecked ?> TYPE="checkbox" NAME="protect"/> Prevent non-admin users from editing the <B><?php echo $QW['page'] ?></B> page</TD></TR><?php
					}
				} ?>
			</TABLE>
		</FORM>
		<?php QWFormatHelp( "<P>To edit the page, modify the text in the edit box above.  See " . 
			QWFormatQwikiPageName( 'QwikiSyntax' ) . " for instructions on how to autoformat
			your source text using wiki syntax rules, or use the following HTML tags: <CODE>" .
			str_replace( "<", "&lt;", $QW_CONFIG['acceptableHTMLTags'] ) .			
			"</CODE>.  Click <B>Save</B> to save your changes and show them in the preview box below.
			When finished, click <B>Done</B> to save your changes and return to the page.</P>
			<UL><LI>Do <B>NOT</B> click the <B>Back</B> button on your Web browser.  Instead,
			if you are done editing, click <B>Done</B> or <B>Cancel</B>.  If you click <B>Back</B>,
			you will leave the page locked and will not be able to edit again until the lock 
			expires.</LI></UL>" .
			( is_file( $QW['mailPath'] ) ?			
				"<UL><LI>Be sure to check the <B>Notify</B> checkbox if you want to notify people 
				on the mailing list that the page has changed.</LI></UL>" :
				"" ) .
			( $QW['userIsAuthenticated'] ? "<UL><LI><B>Admins</B>: Check <B>Prevent non-admin users from editing</B> to require
			logging in with the admin password before allowing editing.</LI></UL>" : "" ) .
			( ini_get( 'file_uploads' ) && $QW_CONFIG['enableAttach'] ? 
			"<P>To add a <B>" . QWCollapseArray( $QW_CONFIG['acceptableExtensions'], ", ", "or " ) . "</B> file attachment to this page, click <B>Browse</B> to select the file
			from your hard drive, and then click <B>Attach</B> to attach it to the page.
			To delete a file attachment, click <B>Delete</B> next to the attachment to
			delete.</P>" : "" ),
				"edit.php?page=$QW[page]" ) ?>
				
		<?php if( ini_get( 'file_uploads' ) && $QW_CONFIG['enableAttach'] ) { ?>
			<FORM CLASS='QWForm' ENCTYPE="multipart/form-data" METHOD="post" ACTION="edit.php">
				<TABLE CLASS='QWInnerSection'>
					<TR CLASS='QWInnerSectionTitle'><TD ALIGN='left'>Filename</TD><TD ALIGN='center'>Size</TD><TD ALIGN='right'></TD></TR>
					<?php
					$numLines = 0;
					if( is_dir( $QW['attachDir'] ) )
					{
						// Read the directory
						$attachDir = dir( $QW['attachDir'] );
						while( $filename = $attachDir->read( ) )
						{
							// Output the result
							if( $filename != "." && $filename != ".." )
							{
								// Output a file line
								++$numLines;
								$attachPath = "$QW[attachDir]/$filename";
								$nameHTML = "<A HREF='$attachPath'$QW[hrefTarget]>$filename</A>";
								$sizeHTML = QWFormatFileSize( filesize( $attachPath ) );
								$commandHTML = "<A HREF='edit.php?page=$QW[page]&action=deletefile&filename=$filename'>delete</A>";
								echo "<TR>" .
									     "<TD ALIGN='left'>$nameHTML</TD>" . 
									     "<TD ALIGN='center'>$sizeHTML</TD>" . 
									     "<TD ALIGN='right'>$commandHTML</TD>" .
								     "</TR>\n";
							}
						}
						$attachDir->close( );
					}
					if( !$numLines )
					{
						// Output that there are no attachments
						echo "<TR><TD ALIGN='center' COLSPAN='2'>(no attachments, Browse to a file and click Upload to attach)</TD></TR>\n";
					}?>
					<INPUT TYPE="hidden" NAME="page" VALUE="<?php echo $QW['page'] ?>"/>
					<INPUT TYPE="hidden" NAME="from" VALUE="<?php echo $QW['requestFrom'] ?>"/>
					<INPUT TYPE="hidden" NAME="debug" VALUE="<?php echo $QW['requestDebug'] ?>"/>
					<INPUT TYPE="hidden" NAME="MAX_FILE_SIZE" VALUE="<?php echo $QW_CONFIG['maxUploadSize'] ?>"/>
					<TR><TD ALIGN='center' COLSPAN='3'><INPUT SIZE="40" TYPE="file" NAME="userfile"/><INPUT TYPE="submit" NAME="action" VALUE="Attach"/></TD></TR>
				</TABLE>
			</FORM>
			<BR/>
		<?php } ?>
		
		<TABLE CLASS='QWInnerSection' WIDTH='80%'>
			<TR CLASS="QWInnerSectionTitle"><TD>Preview</TD></TR>
			<TR><TD><?php echo QWFormatQwikiFile( $QW['pagePath'] ) ?></TD></TR>
		</TABLE>
	</DIV>
	<?php	
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////// Fill Template ///////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

// Include the template
include( $QW_CONFIG['templateFilename'] );

// Output debugging, if requested
if( $QW_CONFIG['enableDebugging'] && $QW['requestDebug'] ) echo QWFormatDebug( );

?>