<?php 
// search.php
//
// Copyright 2004, David Barrett.  All Rights Reserved.
//   Email: dbarrett@quinthar.com
//     Web: http://www.quinthar.com
//
// See LICENSE for the complete licensing details.
require_once( "_global.php" );

////////////////////////////////////////////////////////////////////////////////
///////////////////////////////// Disable Caching //////////////////////////////
////////////////////////////////////////////////////////////////////////////////
QWDisableCaching( );

////////////////////////////////////////////////////////////////////////////////
//////////////////////////// Define Template Functions /////////////////////////
////////////////////////////////////////////////////////////////////////////////

// QWTSearchFormatGlobalNav
$QW_TEMPLATE['getGlobalNavPageNameList'] = "QWTSearchFormatGlobalNav";
function QWTSearchFormatGlobalNav( )
{
	// Output from the config file
	global $QW_CONFIG;
	return QWFormatQwikiText( $QW_CONFIG['globalNav'] );
}

// QWTSearchFormatTitle
$QW_TEMPLATE['getTitle'] = "QWTSearchFormatTitle";
function QWTSearchFormatTitle( )
{
	// Output a title
	global $QW;
	return QWCleanQwikiPageName( "Searching: $QW[requestQuery]" );
}

// QWTSearchInjectBody
$QW_TEMPLATE['injectBody'] = "QWTSearchInjectBody";
function QWTSearchInjectBody( )
{
	// Perform the search
	global $QW, $QW_CONFIG;
	
	// Strip out any non alphanumeric, non-whitespace data
	$safeQuery = preg_replace( "/[^\w\d\s]+/", "", $QW['requestQuery'] );
	if( !$safeQuery ) QWFormatMessage( "Please type a search phrase and click <B>Search</B>" );

	
	// Output a search form
	?>
	<FORM CLASS='QWForm' ENCTYPE="multipart/form-data" METHOD="get" ACTION="search.php">
		<INPUT TYPE="hidden" NAME="debug" VALUE="<?php echo $QW['requestDebug'] ?>"/>
		<INPUT TYPE="hidden" NAME="help" VALUE="<?php echo $QW['requestHelp'] ?>"/>
		<TABLE CLASS='QWInnerSection' ALIGN='center'>
			<TR CLASS='QWInnerSectionTitle'><TD>Search phrase:</TD></TR>
			<TR><TD><INPUT TYPE='text' NAME='query' VALUE='<?php echo $QW['requestQuery'] ?>'/><INPUT TYPE='submit' NAME='action' ACTION='search.php' VALUE='Search'/></TD></TR>
		</TABLE>	
	</FORM>
	<?php 
	QWFormatHelp( "<P>To search for a specific phrase, type the phrase into the <B>Search phrase</B>
		field above and click <B>Search</B>.  This will generate a list of all pages
		containing the exact phrase specified, ignoring case and punctuation.  The results
		list has the following columns:</P>
		<UL><LI><B>#</B>: The number of matches found in the page</LI>
		<LI><B>Page</B>: A link to the page containing the phrase</LI>
		<LI><B>Quote</B>: A short quote of the context surrounding the first match</LI></UL>",
		"search.php?query=$QW[requestQuery]" );

	// Do the query	
	if( strlen( $safeQuery ) )
	{	
		// Compose a pattern from the keywords
		$pattern = "/($safeQuery)/i";
		
		// Loop across all files one by one and grep for the pattern
		if( is_dir( 'data/' ) )
		{
			// Read the directory
			$dataDir = dir( 'data/' );
			while( $filename = $dataDir->read( ) )
			{
				// Output the result
				if( $filename != "." && $filename != ".." )
				{
					// See if it's a qwiki file
					$info = pathinfo( $filename );
					if( isset( $info['extension'] ) &&
					    !strcasecmp( $info['extension'], 'qwiki' ) )
					{
						// Found a qwiki file, do the test
						$path = "data/$filename";
						$length = filesize( $path );
						$fp = fopen( $path, "rb" );
						$data = fread( $fp, $length );
						fclose( $fp );
						$num = preg_match_all( $pattern, $data, $matches );
						if( $num )
						{
							// At least one match, note as much
							$pagename = substr( $filename, 0, -strlen( $info['extension'] )-1 );
							if( !isset( $results[$num] ) ) $c = 0;
							else                           $c = count( $results[$num] );
							$results[ $num ][ $c ][0] = $pagename;
							
							// Grab a quote around the first instance
							$match = $matches[1][0];
							$pos = strpos( $data, $match );
							$width = 40;
							$start = $pos - $width;
							$afterPos = $pos + strlen( $match );
							$end = $afterPos + $width;
							if( $start < 0 ) $start = 0;
							if( $end >= strlen( $data ) ) $end = strlen( $data );
							$before = substr( $data, $start, $pos - $start );
							$after = substr( $data, $afterPos, $end - $afterPos );
							$quote = "$before<B>$match</B>$after"; 
							$results[ $num ][ $c ][1] = $quote;
						}
						else
						{
							// Even if it's not found in the file, check the filename
							$pagename = substr( $filename, 0, -strlen( $info['extension'] )-1 );
							$cleanname = QWCleanQwikiPageName( $pagename );
							if( preg_match( $pattern, $cleanname, $matches ) )
							{
								// Matched the filename, add it without a quote
								if( !isset( $results[0] ) ) $c = 0;
								else                        $c = count( $results[0] );
								$results[ 0 ][ $c ][0] = $pagename;
								$results[ 0 ][ $c ][1] = "(none)";
							}
						}
					}
				}
			}
			$dataDir->close( );
			
			// Output the results, if there are any
			echo "<DIV ALIGN='center'>\n";
			echo "<TABLE CLASS='QWInnerSection'>\n";
			echo "<TR CLASS='QWInnerSectionTitle'><TD>#</TD><TD>Page</TD><TD>Quote</TD></TR>\n";
			if( isset( $results ) && is_array( $results ) && count( $results ) )
			{
				// Sort the results
				krsort( $results );
				
				// Output the results
				foreach( $results as $num => $list )
				{
					// Sort the list of matches with this count and output
					sort( $list );
					for( $c=0; $c<count( $list ); ++$c )
					{
						// Output
						$pagename = $list[$c][0];
						$quote    = $list[$c][1];
						echo "<TR><TD>$num</TD><TD>" . QWFormatQwikiPageName( $pagename ) . "</TD><TD>...$quote...</TD></TR>\n";
					}
				}
			}
			else
			{
				// No matches
				echo "<TR><TD COLSPAN='3'>No matches found</TD></TR>\n";
			}
			echo "</TABLE>\n";
			echo "</DIV>\n";
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////// Fill Template ///////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

// Include the template
include( $QW_CONFIG['templateFilename'] );

// Output debugging, if requested
if( $QW_CONFIG['enableDebugging'] && $QW['requestDebug'] ) echo QWFormatDebug( );

?>